@@ -25,12 +25,16 @@ try:
 except ImportError:
     pass
 
+from utils import no_coverage_env
 import pexpect
 import sys
 
+
 def main():
-    p = pexpect.spawn(sys.executable + ' echo_w_prompt.py')
+    p = pexpect.spawn(sys.executable + ' echo_w_prompt.py',
+                      env=no_coverage_env())
     p.interact()
+    print("Escaped interact")
 
 if __name__ == '__main__':
     main()
